@@ -1,4 +1,4 @@
-(* $Id: tron.ml,v 1.13 2000/06/06 03:51:06 garrigue Exp $ *)
+(* $Id: tron.ml,v 1.14 2001/09/06 08:47:55 garrigue Exp $ *)
 
 (* Tron? Game *)
 open GMain
@@ -14,7 +14,7 @@ let main () =
 (* Game State *)
   let gameSize = 64 in
   let gameState =
-    Array.create_matrix ~dimx:(gameSize+2) ~dimy:(gameSize+2) 0 in
+    Array.create_matrix (gameSize+2) (gameSize+2) 0 in
   let gameInit _ = 
     for i=1 to gameSize do
       for j=1 to gameSize do
