@@ -1,4 +1,6 @@
-(* $Id: test.ml,v 1.1 2000/04/27 10:00:06 garrigue Exp $ *)
+(* $Id: test.ml,v 1.2 2001/09/06 08:47:56 garrigue Exp $ *)
+
+open StdLabels
 
 let test_string2 = String.concat ~sep:"" [
   "<html>\n";
@@ -15,8 +17,8 @@ let test_string2 = String.concat ~sep:"" [
 let read_file file =
   let ic = open_in file in
   let b = Buffer.create 16384 and s = String.create 1024 and len = ref 0 in
-  while len := input ic ~buf:s ~pos:0 ~len:1024; !len > 0 do
-    Buffer.add_substring b s ~pos:0 ~len:!len
+  while len := input' ic ~buf:s ~pos:0 ~len:1024; !len > 0 do
+    Buffer.add_substring b s 0 !len
   done;
   Buffer.contents b
 
