@@ -1,4 +1,4 @@
-(* $Id: gList.mli,v 1.25 2000/06/15 09:52:09 garrigue Exp $ *)
+(* $Id: gList.mli,v 1.27 2001/10/09 08:43:56 garrigue Exp $ *)
 
 open Gtk
 open GObj
@@ -22,12 +22,22 @@ val list_item :
   ?height:int ->
   ?packing:(list_item -> unit) -> ?show:bool -> unit -> list_item
 
+class liste_signals : Gtk.liste obj ->
+  object
+    inherit container_signals
+    val obj : Gtk.liste obj
+    method select_child : callback:(list_item -> unit) -> GtkSignal.id
+    method selection_changed : callback:(unit -> unit) -> GtkSignal.id
+    method unselect_child : callback:(list_item -> unit) -> GtkSignal.id
+  end
+
 class liste : Gtk.liste obj ->
   object
     inherit [list_item] item_container
     val obj : Gtk.liste obj
     method child_position : list_item -> int
     method clear_items : start:int -> stop:int -> unit
+    method connect : liste_signals
     method insert : list_item -> pos:int -> unit
     method select_item : pos:int -> unit
     method unselect_item : pos:int -> unit
@@ -137,6 +147,26 @@ class ['a] clist : Gtk.clist obj ->
     method vadjustment : GData.adjustment
   end
 val clist :
+  ?columns:int ->
+  ?titles:string list ->
+  ?hadjustment:GData.adjustment ->
+  ?vadjustment:GData.adjustment ->
+  ?shadow_type:Tags.shadow_type ->
+  ?button_actions:(int * Tags.button_action list) list ->
+  ?selection_mode:Tags.selection_mode ->
+  ?reorderable:bool ->
+  ?use_drag_icons:bool ->
+  ?row_height:int ->
+  ?titles_show:bool ->
+  ?titles_active:bool ->
+  ?auto_sort:bool ->
+  ?sort_column:int ->
+  ?sort_type:Tags.sort_type ->
+  ?border_width:int ->
+  ?width:int ->
+  ?height:int ->
+  ?packing:(widget -> unit) -> ?show:bool -> unit -> string clist
+val clist_poly :
   ?columns:int ->
   ?titles:string list ->
   ?hadjustment:GData.adjustment ->
