@@ -1,5 +1,6 @@
-(* $Id: gObj.ml,v 1.34 2001/03/12 03:43:42 garrigue Exp $ *)
+(* $Id: gObj.ml,v 1.37 2001/12/12 10:53:50 garrigue Exp $ *)
 
+open StdLabels
 open Gaux
 open Gtk
 open GtkData
@@ -263,9 +264,7 @@ and widget obj = object (self)
   method as_widget = (obj :> Gtk.widget obj)
   method misc = new misc_ops (obj :> Gtk.widget obj)
   method drag = new drag_ops (Object.unsafe_cast obj : Gtk.widget obj)
-  method coerce =
-    (self :> < destroy : _; get_id : _; as_widget : _; misc : _;
-               drag : _; coerce : _ >)
+  method coerce = (self :> widget)
 end
 
 (* just to check that GDraw.misc_ops is compatible with misc_ops *)
@@ -274,13 +273,6 @@ let _ = fun (x : #GDraw.misc_ops) -> (x : misc_ops)
 class widget_signals ?after (obj : [> `widget] obj) =
   gtkobj_signals ?after obj
 
-(*
-class widget_coerce obj = object
-  inherit widget obj
-  method coerce = (self :> widget)
-end
-*)
-
 class widget_full obj = object
   inherit widget obj
   method connect = new widget_signals obj
