@@ -1,5 +1,6 @@
-(* $Id: gdk.ml,v 1.44 2001/03/12 03:43:43 garrigue Exp $ *)
+(* $Id: gdk.ml,v 1.49 2001/10/04 09:45:40 garrigue Exp $ *)
 
+open StdLabels
 open Gaux
 
 type colormap
@@ -143,6 +144,8 @@ module Color = struct
   external colormap_new : visual -> privat:bool -> colormap
       = "ml_gdk_colormap_new"
   let get_colormap ?(privat=false) vis = colormap_new vis ~privat
+  external get_visual : colormap -> visual
+      = "ml_gdk_colormap_get_visual"
 
   type spec = [ `BLACK | `NAME of string | `RGB of int * int * int | `WHITE]
   let color_alloc ~colormap color =
@@ -176,6 +179,7 @@ module Window = struct
   type background_pixmap = [ `NONE | `PARENT_RELATIVE | `PIXMAP of pixmap]
   external visual_depth : visual -> int = "ml_gdk_visual_get_depth"
   external get_visual : window -> visual = "ml_gdk_window_get_visual"
+  external get_colormap : window -> colormap = "ml_gdk_window_get_colormap"
   external get_parent : window -> window = "ml_gdk_window_get_parent"
   external get_size : 'a drawable -> int * int = "ml_gdk_window_get_size"
   external get_position : 'a drawable -> int * int =
@@ -204,6 +208,15 @@ module PointArray = struct
     set arr ~pos
 end
 
+module SegmentArray = struct
+  type t = { len: int}
+  external create : len:int -> t = "ml_segment_array_new"
+  external set : t -> pos:int -> x1:int -> y1:int -> x2:int -> y2: int -> unit = "ml_segment_array_set_bc" "ml_segment_array_set"
+  let set arr ~pos =
+    if pos < 0 || pos >= arr.len then invalid_arg "SegmentArray.set";
+    set arr ~pos
+end
+
 module Region = struct
   type gdkFillRule = [ `EVEN_ODD_RULE|`WINDING_RULE ]
   type gdkOverlapType = [ `IN|`OUT|`PART ]
@@ -270,6 +283,8 @@ module GC = struct
       gc -> width:int -> style:gdkLineStyle -> cap:gdkCapStyle ->
       join:gdkJoinStyle -> unit
       = "ml_gdk_gc_set_line_attributes"
+  external set_dashes : gc -> offset:int -> int list -> unit =
+    "ml_gdk_gc_set_dashes"
   external copy : dst:gc -> gc -> unit = "ml_gdk_gc_copy"
   type values = {
       foreground : Color.t;
@@ -295,28 +310,40 @@ module GC = struct
 end
 
 module Pixmap = struct
-  external create : window -> width:int -> height:int -> depth:int -> pixmap
+  open Gpointer
+  external create :
+      window optboxed -> width:int -> height:int -> depth:int -> pixmap
       = "ml_gdk_pixmap_new"
+  let create ?window ~width ~height ?(depth = -1) () =
+    try create (optboxed window) ~width ~height ~depth
+    with _ -> failwith "Gdk.Pixmap.create"
   external create_from_data :
-      window -> string -> width:int -> height:int -> depth:int ->
+      window optboxed -> string -> width:int -> height:int -> depth:int ->
       fg:Color.t -> bg:Color.t -> pixmap
       = "ml_gdk_pixmap_create_from_data_bc" "ml_gdk_pixmap_create_from_data"
+  let create_from_data ?window ~width ~height ?(depth = -1) ~fg ~bg data =
+    try create_from_data (optboxed window) data ~width ~height ~depth ~fg ~bg
+    with _ -> failwith "Gdk.Pixmap.create_from_data"
   external create_from_xpm :
-      window -> ?colormap:colormap -> ?transparent:Color.t ->
+      ?window:window -> ?colormap:colormap -> ?transparent:Color.t ->
       file:string -> unit -> pixmap * bitmap
       = "ml_gdk_pixmap_colormap_create_from_xpm"
   external create_from_xpm_d :
-      window -> ?colormap:colormap -> ?transparent:Color.t ->
+      ?window:window -> ?colormap:colormap -> ?transparent:Color.t ->
       data:string array -> unit -> pixmap * bitmap
       = "ml_gdk_pixmap_colormap_create_from_xpm_d"
 end
 
 module Bitmap = struct
-  let create : window -> width:int -> height:int -> bitmap =
-    Obj.magic (Pixmap.create ~depth:1)
+  open Gpointer
+  let create ?window ~width ~height () : bitmap =
+    Obj.magic (Pixmap.create ?window ~width ~height ~depth:1 () : pixmap)
   external create_from_data :
-      window -> string -> width:int -> height:int -> bitmap
+      window optboxed -> string -> width:int -> height:int -> bitmap
       = "ml_gdk_bitmap_create_from_data"
+  let create_from_data ?window ~width ~height data =
+    try create_from_data (optboxed window) data ~width ~height
+    with _ -> failwith "Gdk.Bitmap.create_from_data"
 end
 
 module Font = struct
@@ -353,14 +380,32 @@ module Draw = struct
     arc w gc ~x ~y ~width ~height ~filled
       ~start:(truncate(start *. 64.))
       ~angle:(truncate(angle *. 64.))
+
+  let f_pointarray f l = 
+    let array_of_points l =
+      let len = List.length l in
+      let arr = PointArray.create ~len in
+      List.fold_left l ~init:0
+      	~f:(fun pos (x,y) -> PointArray.set arr ~pos ~x ~y; pos+1);
+      arr
+    in
+    f (array_of_points l)
+
+  let f_segmentarray f l = 
+    let array_of_segments l =
+      let len = List.length l in
+      let arr = SegmentArray.create ~len in
+      List.fold_left l ~init:0
+      	~f:(fun pos ((x1,y1),(x2,y2)) -> 
+	  SegmentArray.set arr ~pos ~x1 ~y1 ~x2 ~y2; pos+1);
+      arr
+    in
+    f (array_of_segments l)
+
   external polygon : 'a drawable -> gc -> filled:bool -> PointArray.t -> unit
       = "ml_gdk_draw_polygon"
   let polygon w gc ?(filled=false) l =
-    let len = List.length l in
-    let arr = PointArray.create ~len in
-    List.fold_left l ~init:0
-      ~f:(fun pos (x,y) -> PointArray.set arr ~pos ~x ~y; pos+1);
-    polygon w gc ~filled arr
+    f_pointarray (polygon w gc ~filled) l
   external string : 'a drawable -> font: font -> gc -> x: int -> y: int ->
     string: string -> unit
       = "ml_gdk_draw_string_bc" "ml_gdk_draw_string"	
@@ -378,6 +423,16 @@ module Draw = struct
     xsrc: int -> ysrc: int -> xdest: int -> ydest: int -> 
     width: int -> height: int -> unit
       = "ml_gdk_draw_pixmap_bc" "ml_gdk_draw_pixmap"
+
+  external points : 'a drawable -> gc -> PointArray.t -> unit
+      = "ml_gdk_draw_points"
+  let points w gc l = f_pointarray (points w gc) l
+  external lines : 'a drawable -> gc -> PointArray.t -> unit
+      = "ml_gdk_draw_lines"
+  let lines w gc l = f_pointarray (lines w gc) l
+  external segments : 'a drawable -> gc -> SegmentArray.t -> unit
+      = "ml_gdk_draw_segments"
+  let segments w gc l = f_segmentarray (segments w gc) l
 end
 
 module Rgb = struct
@@ -420,13 +475,13 @@ module Truecolor = struct
     match Visual.get_type visual with
       `TRUE_COLOR | `DIRECT_COLOR ->
 	let shift_prec = shift_prec visual in
-	Format.eprintf "red : %d %d, "
+        (* Format.eprintf "red : %d %d, "
 	  shift_prec.red_shift shift_prec.red_prec;
 	Format.eprintf "green : %d %d, "
 	  shift_prec.green_shift shift_prec.green_prec;
 	Format.eprintf "blue : %d %d"
 	  shift_prec.blue_shift shift_prec.blue_prec;
-	Format.pp_print_newline Format.err_formatter ();
+	Format.pp_print_newline Format.err_formatter (); *)
 	let red_lsr = 16 - shift_prec.red_prec
 	and green_lsr = 16 - shift_prec.green_prec
 	and blue_lsr = 16 - shift_prec.blue_prec in
