@@ -1,4 +1,4 @@
-/* $Id: ml_gdk.c,v 1.43 2001/02/27 09:52:11 garrigue Exp $ */
+/* $Id: ml_gdk.c,v 1.46 2001/10/04 09:45:40 garrigue Exp $ */
 
 #include <string.h>
 #include <gdk/gdk.h>
@@ -76,11 +76,13 @@ Make_Extractor (GdkVisual,GdkVisual_val,blue_prec,Val_int)
 
 /* Image */
 
+#ifndef UnsafeImage
+
 Make_Val_final_pointer (GdkImage, Ignore, gdk_image_destroy, 0)
 GdkImage *GdkImage_val(value val)
 {
-    if (!Field(val,1)) ml_raise_gdk ("attempt to use destroyed GdkImage");
-    return (GdkImage*)(Field(val,1));
+  if (!Field(val,1)) ml_raise_gdk ("attempt to use destroyed GdkImage");
+  return (GdkImage*)(Field(val,1));
 }
 value ml_gdk_image_destroy (value val)
 {
@@ -88,6 +90,21 @@ value ml_gdk_image_destroy (value val)
     Field(val,1) = 0;
     return Val_unit;
 }
+
+#else
+
+/* Unsafe Image */
+
+#define GdkImage_val(val) ((GdkImage*)val)
+#define Val_GdkImage(img) ((value) img)
+value ml_gdk_image_destroy (value val)
+{
+  gdk_image_destroy(GdkImage_val(val));
+  return Val_unit;
+}
+
+#endif
+
 ML_4 (gdk_image_new_bitmap, GdkVisual_val, String_val, Int_val, Int_val,
       Val_GdkImage)
 ML_4 (gdk_image_new, GdkImageType_val, GdkVisual_val, Int_val, Int_val,
@@ -100,6 +117,7 @@ Make_Extractor(gdk_image, GdkImage_val, visual, Val_GdkVisual)
 Make_Extractor(gdk_image, GdkImage_val, width, Val_int)
 Make_Extractor(gdk_image, GdkImage_val, height, Val_int)
 Make_Extractor(gdk_image, GdkImage_val, depth, Val_int)
+
 /*
 Make_Extractor(gdk_image, GdkImage_val, bpp, Val_int)
 Make_Extractor(gdk_image, GdkImage_val, bpl, Val_int)
@@ -109,6 +127,7 @@ Make_Extractor(gdk_image, GdkImage_val, mem, Val_pointer)
 /* Color */
 
 ML_2 (gdk_colormap_new, GdkVisual_val, Bool_val, Val_GdkColormap)
+ML_1 (gdk_colormap_get_visual, GdkColormap_val, Val_GdkVisual)
 
 value ml_gdk_color_white (value cmap)
 {
@@ -171,6 +190,7 @@ Make_Extractor (GdkRectangle, GdkRectangle_val, height, Val_int)
 Make_Val_final_pointer (GdkWindow, gdk_window_ref, gdk_window_unref, 0)
 Make_Extractor (gdk_visual_get, GdkVisual_val, depth, Val_int)
 ML_1 (gdk_window_get_visual, GdkWindow_val, Val_GdkVisual)
+ML_1 (gdk_window_get_colormap, GdkWindow_val, Val_GdkColormap)
 ML_3 (gdk_window_set_back_pixmap, GdkWindow_val, GdkPixmap_val, Int_val, Unit)
 ML_1 (gdk_window_clear, GdkWindow_val, Unit)
 ML_0 (GDK_ROOT_PARENT, Val_GdkWindow)
@@ -229,14 +249,17 @@ value ml_gdk_pixmap_colormap_create_from_xpm
 	(value window, value colormap, value transparent, char *filename)
 {
     CAMLparam0();
-    GdkBitmap *mask;
+    GdkPixmap *pixmap;
+    GdkBitmap *mask = NULL;
     CAMLlocal2(vpixmap, vmask);
     value ret;
 
-    vpixmap = Val_GdkPixmap_no_ref
-	(gdk_pixmap_colormap_create_from_xpm
-	 (GdkWindow_val(window), Option_val(colormap,GdkColormap_val,NULL),
-	  &mask, Option_val(transparent,GdkColor_val,NULL), filename));
+    pixmap = gdk_pixmap_colormap_create_from_xpm
+        (Option_val(window,GdkWindow_val,NULL),
+         Option_val(colormap,GdkColormap_val,NULL),
+         &mask, Option_val(transparent,GdkColor_val,NULL), filename);
+    if (!pixmap) ml_raise_gdk ("Gdk.Pixmap.create_from_xpm_file");
+    vpixmap = Val_GdkPixmap_no_ref(pixmap);
     vmask = Val_GdkBitmap_no_ref (mask);
 
     ret = alloc_small (2,0);
@@ -249,14 +272,17 @@ value ml_gdk_pixmap_colormap_create_from_xpm_d
 	(value window, value colormap, value transparent, char **data)
 {
     CAMLparam0();
-    GdkBitmap *mask;
+    GdkPixmap *pixmap;
+    GdkBitmap *mask = NULL;
     CAMLlocal2(vpixmap, vmask);
     value ret;
 
-    vpixmap = Val_GdkPixmap_no_ref
-	(gdk_pixmap_colormap_create_from_xpm_d
-	 (GdkWindow_val(window), Option_val(colormap,GdkColormap_val,NULL),
-	  &mask, Option_val(transparent,GdkColor_val,NULL), data));
+    pixmap = gdk_pixmap_colormap_create_from_xpm_d
+        (Option_val(window,GdkWindow_val,NULL),
+         Option_val(colormap,GdkColormap_val,NULL),
+         &mask, Option_val(transparent,GdkColor_val,NULL), data);
+    if (!pixmap) ml_raise_gdk ("Gdk.Pixmap.create_from_xpm_data");
+    vpixmap = Val_GdkPixmap_no_ref (pixmap);
     vmask = Val_GdkBitmap_no_ref (mask);
 
     ret = alloc_small (2, 0);
@@ -336,6 +362,32 @@ ML_2 (gdk_gc_set_subwindow, GdkGC_val, GdkSubwindowMode_val, Unit)
 ML_2 (gdk_gc_set_exposures, GdkGC_val, Bool_val, Unit)
 ML_5 (gdk_gc_set_line_attributes, GdkGC_val, Int_val, GdkLineStyle_val,
       GdkCapStyle_val, GdkJoinStyle_val, Unit)
+
+value ml_gdk_gc_set_dashes(value gc, value offset, value dashes)
+{
+  CAMLparam3(gc, offset, dashes);
+  CAMLlocal1(tmp);
+  int l = 0;
+  int i;
+  char *cdashes;
+  for(tmp = dashes; tmp != Val_int(0); tmp = Field(tmp,1)){
+    l++;
+  }
+  if( l == 0 ){ ml_raise_gdk("line dashes must have at least one element"); }
+  cdashes = malloc(sizeof(char) * l);
+  for(i=0, tmp= dashes; i<l; i++, tmp = Field(tmp,1)){
+    int d;
+    d = Int_val(Field(tmp,0));
+    if( d<0 && d>255 ){
+      ml_raise_gdk("line dashes must be [0..255]");
+    }
+    cdashes[i] = (char)d;
+  }
+  gdk_gc_set_dashes( GdkGC_val(gc), Int_val(offset), cdashes, l);
+  CAMLreturn(Val_unit);
+}
+  
+
 ML_2 (gdk_gc_copy, GdkGC_val, GdkGC_val, Unit)
 value ml_gdk_gc_get_values (value gc)
 {
@@ -396,6 +448,26 @@ value ml_point_array_set (value arr, value pos, value x, value y)
     return Val_unit;
 }
 
+#define SegmentArray_val(val) ((GdkSegment*)&Field(val,1))
+#define SegmentArrayLen_val(val) Int_val(Field(val,0))
+value ml_segment_array_new (value len)
+{
+    value ret = alloc (1 + Wosize_asize(Int_val(len)*sizeof(GdkSegment)),
+		       Abstract_tag);
+    Field(ret,0) = len;
+    return ret;
+}
+value ml_segment_array_set (value arr, value pos, value x1, value y1, value x2, value y2)
+{
+    GdkSegment *pt = SegmentArray_val(arr) + Int_val(pos);
+    pt->x1 = Int_val(x1);
+    pt->y1 = Int_val(y1);
+    pt->x2 = Int_val(x2);
+    pt->y2 = Int_val(y2);
+    return Val_unit;
+}
+ML_bc6 (ml_segment_array_set)
+
 ML_4 (gdk_draw_point, GdkDrawable_val, GdkGC_val, Int_val, Int_val, Unit)
 ML_6 (gdk_draw_line, GdkDrawable_val, GdkGC_val, Int_val, Int_val,
       Int_val, Int_val, Unit)
@@ -419,6 +491,12 @@ ML_9 (gdk_draw_pixmap, GdkDrawable_val, GdkGC_val, GdkPixmap_val, Int_val, Int_v
 ML_bc9 (ml_gdk_draw_pixmap)
 ML_9 (gdk_draw_image, GdkDrawable_val, GdkGC_val, GdkImage_val, Int_val, Int_val, Int_val, Int_val, Int_val, Int_val, Unit)
 ML_bc9 (ml_gdk_draw_image)
+ML_3 (gdk_draw_points, GdkDrawable_val, GdkGC_val, 
+      Insert(PointArray_val(arg3)) PointArrayLen_val, Unit)
+ML_3 (gdk_draw_segments, GdkDrawable_val, GdkGC_val, 
+      Insert(SegmentArray_val(arg3)) SegmentArrayLen_val, Unit)
+ML_3 (gdk_draw_lines, GdkDrawable_val, GdkGC_val, 
+      Insert(PointArray_val(arg3)) PointArrayLen_val, Unit)
 
 /* RGB */
 
