@@ -1,4 +1,4 @@
-/* $Id: ml_gtk.c,v 1.99 2001/02/20 23:26:38 garrigue Exp $ */
+/* $Id: ml_gtk.c,v 1.102 2001/12/10 06:49:08 garrigue Exp $ */
 
 #include <string.h>
 #include <gtk/gtk.h>
@@ -8,6 +8,10 @@
 #include <caml/callback.h>
 #include <caml/fail.h>
 
+#if (GTK_MAJOR_VERSION <= 1) && (GTK_MINOR_VERSION <= 2) && (GTK_MICRO_VERSION <= 3)
+#define GTK_WIN_POS_CENTER_ALWAYS 0
+#endif
+
 #include "wrappers.h"
 #include "ml_glib.h"
 #include "ml_gdk.h"
@@ -155,6 +159,21 @@ Make_Extractor (gtk_adjustment_get, GtkAdjustment_val, step_increment,
 Make_Extractor (gtk_adjustment_get, GtkAdjustment_val, page_increment,
 		copy_double)
 Make_Extractor (gtk_adjustment_get, GtkAdjustment_val, page_size, copy_double)
+value ml_gtk_adjustment_set(value lower, value upper,
+                            value step_increment, value page_increment,
+                            value page_size, value adjustment)
+{
+    GtkAdjustment *adj = GtkAdjustment_val(adjustment);
+#define Update_field(name) adj->name = Option_val(name,Double_val,adj->name)
+    Update_field(lower);
+    Update_field(upper);
+    Update_field(step_increment);
+    Update_field(page_increment);
+    Update_field(page_size);
+#undef Update_field
+    return Val_unit;
+}
+ML_bc6(ml_gtk_adjustment_set)
 
 /* gtktooltips.h */
 
@@ -280,7 +299,7 @@ static value Val_GtkAllocation (GtkAllocation allocation)
     return ret;
 }
 Make_Extractor (gtk_widget, GtkWidget_val, allocation, Val_GtkAllocation)
-ML_1(Val_GtkAllocation, *(GtkAllocation*)Pointer_val, )
+ML_1(Val_GtkAllocation, *(GtkAllocation*)Pointer_val, (value))
 /*
 #define GtkAllocation_val(val) ((GtkAllocation*)Pointer_val(val))
 Make_Extractor (gtk_allocation, GtkAllocation_val, x, Val_int)
